@@ -1,4 +1,6 @@
-from typing import Union, NamedTuple, Any, List, Optional
+from dataclasses import dataclass, replace
+from typing import Union, Any, List, Optional
+from warnings import warn
 
 import numpy as np
 from numpy import ndarray
@@ -7,7 +9,8 @@ from skfem.element import Element
 from skfem.mesh import Mesh
 
 
-class DofsView(NamedTuple):
+@dataclass(repr=False)
+class DofsView:
     """A subset of :class:`skfem.assembly.Dofs`."""
 
     obj: Any = None
@@ -20,6 +23,54 @@ class DofsView(NamedTuple):
     edge_rows: Union[ndarray, slice] = slice(None)
     interior_rows: Union[ndarray, slice] = slice(None)
 
+    def __repr__(self):
+        nnodal = len(np.unique(
+            self.obj.nodal_dofs[self.nodal_rows][:, self.nodal_ix]
+        ))
+        nfacet = len(np.unique(
+            self.obj.facet_dofs[self.facet_rows][:, self.facet_ix]
+        ))
+        nedge = len(np.unique(
+            self.obj.edge_dofs[self.edge_rows][:, self.edge_ix]
+        ))
+        ninterior = len(np.unique(
+            self.obj.interior_dofs[self.interior_rows][:, self.interior_ix]
+        ))
+        dofnames = np.array(self.obj.element.dofnames)
+        rep = ""
+        rep += "<skfem {}({}, {}) object>".format(
+            type(self).__name__,
+            type(self.obj.topo).__name__,
+            type(self.obj.element).__name__
+        )
+        if nnodal > 0:
+            rep += "\n  Number of nodal DOFs: {} {}".format(
+                nnodal,
+                dofnames[self.nodal_rows]
+            )
+        if nfacet > 0:
+            rep += "\n  Number of facet DOFs: {} {}".format(
+                nfacet,
+                dofnames[self.obj.nodal_dofs.shape[0]:][self.facet_rows],
+            )
+        if nedge > 0:
+            rep += "\n  Number of edge DOFs: {} {}".format(
+                nedge,
+                dofnames[(self.obj.nodal_dofs.shape[0]
+                          + self.obj.facet_dofs.shape[0]):][self.edge_rows],
+            )
+        if ninterior > 0:
+            rep += "\n  Number of interior DOFs: {} {}".format(
+                ninterior,
+                dofnames[(self.obj.nodal_dofs.shape[0]
+                          + self.obj.facet_dofs.shape[0]
+                          + self.obj.edge_dofs.shape[0]):][self.interior_rows],
+            )
+        return rep
+
+    def __str__(self):
+        return self.__repr__()
+
     def flatten(self) -> ndarray:
         """Return all DOF indices as a single array."""
         return np.unique(
@@ -62,19 +113,21 @@ class DofsView(NamedTuple):
             An array of DOF names, e.g. `["u", "u_n"]`.
 
         """
-        return DofsView(
-            self.obj,
-            self.nodal_ix,
-            self.facet_ix,
-            self.edge_ix,
-            self.interior_ix,
-            *self._intersect_tuples(
-                (self.nodal_rows,
-                 self.facet_rows,
-                 self.edge_rows,
-                 self.interior_rows),
-                self._dofnames_to_rows(dofnames)
-            )
+        nrows = self._intersect_tuples(
+            (
+                self.nodal_rows,
+                self.facet_rows,
+                self.edge_rows,
+                self.interior_rows,
+            ),
+            self._dofnames_to_rows(dofnames)
+        )
+        return replace(
+            self,
+            nodal_rows=nrows[0],
+            facet_rows=nrows[1],
+            edge_rows=nrows[2],
+            interior_rows=nrows[3],
         )
 
     def drop(self, dofnames):
@@ -86,19 +139,21 @@ class DofsView(NamedTuple):
             An array of DOF names, e.g. `["u", "u_n"]`.
 
         """
-        return DofsView(
-            self.obj,
-            self.nodal_ix,
-            self.facet_ix,
-            self.edge_ix,
-            self.interior_ix,
-            *self._intersect_tuples(
-                (self.nodal_rows,
-                 self.facet_rows,
-                 self.edge_rows,
-                 self.interior_rows),
-                self._dofnames_to_rows(dofnames, skip=True)
-            )
+        nrows = self._intersect_tuples(
+            (
+                self.nodal_rows,
+                self.facet_rows,
+                self.edge_rows,
+                self.interior_rows,
+            ),
+            self._dofnames_to_rows(dofnames, skip=True)
+        )
+        return replace(
+            self,
+            nodal_rows=nrows[0],
+            facet_rows=nrows[1],
+            edge_rows=nrows[2],
+            interior_rows=nrows[3],
         )
 
     def all(self, key=None):
@@ -143,13 +198,13 @@ class DofsView(NamedTuple):
         return getattr(self.obj, attr)
 
     def __or__(self, other):
-        """For merging two sets of DOFs."""
-        return DofsView(
-            self.obj,
-            np.union1d(self.nodal_ix, other.nodal_ix),
-            np.union1d(self.facet_ix, other.facet_ix),
-            np.union1d(self.edge_ix, other.edge_ix),
-            np.union1d(self.interior_ix, other.interior_ix)
+        warn("Use numpy.hstack to combine sets of DOFs", DeprecationWarning)
+        return replace(
+            self,
+            nodal_ix=np.union1d(self.nodal_ix, other.nodal_ix),
+            facet_ix=np.union1d(self.facet_ix, other.facet_ix),
+            edge_ix=np.union1d(self.edge_ix, other.edge_ix),
+            interior_ix=np.union1d(self.interior_ix, other.interior_ix),
         )
 
     def __add__(self, other):
@@ -170,7 +225,7 @@ class Dofs:
     topo: Mesh
     element: Element
 
-    def __init__(self, topo, element):
+    def __init__(self, topo, element, offset=0):
 
         self.topo = topo
         self.element = element
@@ -178,8 +233,8 @@ class Dofs:
         self.nodal_dofs = np.reshape(
             np.arange(element.nodal_dofs * topo.nvertices, dtype=np.int64),
             (element.nodal_dofs, topo.nvertices),
-            order='F')
-        offset = element.nodal_dofs * topo.nvertices
+            order='F') + offset
+        offset += element.nodal_dofs * topo.nvertices
 
         # edge dofs
         if element.dim == 3 and element.edge_dofs > 0:
@@ -242,6 +297,42 @@ class Dofs:
         # total dofs
         self.N = np.max(self.element_dofs) + 1
 
+    def get_element_dofs(self,
+                         elements: ndarray,
+                         skip_dofnames: List[str] = None) -> DofsView:
+        """Return a subset of DOFs corresponding to the given elements.
+
+        Parameters
+        ----------
+        elements
+            An array of element indices.
+        skip_dofnames
+            An array of dofnames to skip.
+
+        """
+        nodal_ix = (np.empty((0,), dtype=np.int64)
+                    if self.element.nodal_dofs == 0
+                    else np.unique(self.topo.t[:, elements]))
+        edge_ix = (np.empty((0,), dtype=np.int64)
+                   if self.element.edge_dofs == 0
+                   else np.unique(self.topo.t2e[:, elements]))
+        facet_ix = (np.empty((0,), dtype=np.int64)
+                    if self.element.facet_dofs == 0
+                    else np.unique(self.topo.t2f[:, elements]))
+        interior_ix = elements
+
+        if skip_dofnames is None:
+            skip_dofnames = []
+
+        return DofsView(
+            self,
+            nodal_ix,
+            facet_ix,
+            edge_ix,
+            interior_ix,
+            *self._dofnames_to_rows(skip_dofnames, skip=True)
+        )
+
     def get_facet_dofs(self,
                        facets: ndarray,
                        skip_dofnames: List[str] = None) -> DofsView:
@@ -344,8 +435,8 @@ class Dofs:
                 interior_rows.append(i)
 
         return (
-            nodal_rows if len(nodal_rows) > 0 else slice(0, 0),
-            facet_rows if len(facet_rows) > 0 else slice(0, 0),
-            edge_rows if len(edge_rows) > 0 else slice(0, 0),
-            interior_rows if len(interior_rows) > 0 else slice(0, 0)
+            np.array(nodal_rows) if len(nodal_rows) > 0 else slice(0, 0),
+            np.array(facet_rows) if len(facet_rows) > 0 else slice(0, 0),
+            np.array(edge_rows) if len(edge_rows) > 0 else slice(0, 0),
+            np.array(interior_rows) if len(interior_rows) > 0 else slice(0, 0)
         )
