@@ -1,21 +1,36 @@
 """Helper functions for defining forms."""
 
-from typing import Union
+from typing import Union, Optional
 import numpy as np
 from numpy import ndarray, zeros_like
 from skfem.element import DiscreteField
+from skfem.assembly.form.form import FormExtraParams
 
 
 FieldOrArray = Union[DiscreteField, ndarray]
 
 
+def jump(w: FormExtraParams, *args):
+    if not hasattr(w, 'idx'):
+        raise NotImplementedError("jump() can be used only if the form is "
+                                  "assembled through asm().")
+    out = []
+    for i, arg in enumerate(args):
+        out.append((-1.) ** w.idx[i] * arg)
+    return out[0] if len(out) == 1 else tuple(out)
+
+
 def grad(u: DiscreteField):
     """Gradient."""
+    if u.is_zero():
+        return u
     return u.grad
 
 
 def div(u: DiscreteField):
     """Divergence."""
+    if u.is_zero():
+        return u
     if u.div is not None:
         return u.div
     elif u.grad is not None:
@@ -28,6 +43,8 @@ def div(u: DiscreteField):
 
 def curl(u: DiscreteField):
     """Curl."""
+    if u.is_zero():
+        return u
     if u.curl is not None:
         return u.curl
     elif u.grad is not None:
@@ -38,6 +55,8 @@ def curl(u: DiscreteField):
 
 def d(u: DiscreteField):
     """Gradient, divergence or curl."""
+    if u.is_zero():
+        return u
     if u.grad is not None:
         return u.grad
     elif u.div is not None:
@@ -49,41 +68,69 @@ def d(u: DiscreteField):
 
 def sym_grad(u: DiscreteField):
     """Symmetric gradient."""
+    if u.is_zero():
+        return u
     return .5 * (u.grad + transpose(u.grad))
 
 
 def dd(u: DiscreteField):
     """Hessian (if available)."""
+    if u.is_zero():
+        return u
     return u.hess
 
 
-def ddd(u):
+def ddd(u: DiscreteField):
     """Third derivative (if available)."""
+    if u.is_zero():
+        return u
     return u.grad3
 
 
-def dddd(u):
+def dddd(u: DiscreteField):
     """Fourth derivative (if available)."""
+    if u.is_zero():
+        return u
     return u.grad4
 
 
 def dot(u: FieldOrArray, v: FieldOrArray):
     """Dot product."""
+    if isinstance(u, DiscreteField) and u.is_zero():
+        return u
+    if isinstance(v, DiscreteField) and v.is_zero():
+        return v
     return np.einsum('i...,i...', u, v)
 
 
 def ddot(u: FieldOrArray, v: FieldOrArray):
     """Double dot product."""
+    if isinstance(u, DiscreteField) and u.is_zero():
+        return u
+    if isinstance(v, DiscreteField) and v.is_zero():
+        return v
     return np.einsum('ij...,ij...', u, v)
 
 
 def dddot(u: FieldOrArray, v: FieldOrArray):
     """Triple dot product."""
+    if isinstance(u, DiscreteField) and u.is_zero():
+        return u
+    if isinstance(v, DiscreteField) and v.is_zero():
+        return v
     return np.einsum('ijk...,ijk...', u, v)
 
 
-def prod(u: FieldOrArray, v: FieldOrArray, w: FieldOrArray = None):
+def prod(u: FieldOrArray,
+         v: FieldOrArray,
+         w: Optional[FieldOrArray] = None):
     """Tensor product."""
+    if isinstance(u, DiscreteField) and u.is_zero():
+        return u
+    if isinstance(v, DiscreteField) and v.is_zero():
+        return v
+    if isinstance(w, DiscreteField) and w.is_zero():
+        return w
     if w is None:
         return np.einsum('i...,j...->ij...', u, v)
     return np.einsum('i...,j...,k...->ijk...', u, v, w)
