@@ -1,6 +1,8 @@
+import logging
+import importlib
+
 from dataclasses import dataclass, replace
 from typing import Callable, Dict, List, Optional, Tuple, Type, Union
-from warnings import warn
 
 import numpy as np
 from numpy import ndarray
@@ -8,6 +10,9 @@ from numpy import ndarray
 from ..element import BOUNDARY_ELEMENT_MAP, Element
 
 
+logger = logging.getLogger(__name__)
+
+
 @dataclass(repr=False)
 class Mesh:
     """A mesh defined by :class:`~skfem.element.Element` class.
@@ -30,6 +35,7 @@ class Mesh:
     # However, some algorithms (e.g., adaptive refinement) require switching
     # off this behaviour and, hence, this flag exists.
     sort_t: bool = False
+    validate: bool = True  # run validation check if log_level<=DEBUG
 
     @property
     def p(self):
@@ -182,7 +188,7 @@ class Mesh:
                 **({} if self._subdomains is None else self._subdomains),
                 **{name: self.elements_satisfying(test)
                    for name, test in subdomains.items()},
-            }
+            },
         )
 
     def _encode_point_data(self) -> Dict[str, List[ndarray]]:
@@ -241,10 +247,10 @@ class Mesh:
             if subnames[1] == "s":
                 subdomains[subnames[2]] = np.nonzero(data[0])[0]
             elif subnames[1] == "b":
-                boundaries[subnames[2]] = self.t2f[
+                boundaries[subnames[2]] = np.sort(self.t2f[
                     (1 << np.arange(self.t2f.shape[0]))[:, None]
                     & data[0].astype(np.int64) > 0
-                ]
+                ])
 
         return boundaries, subdomains
 
@@ -402,15 +408,92 @@ class Mesh:
 
         # C_CONTIGUOUS is more performant in dimension-based slices
         if not self.doflocs.flags['C_CONTIGUOUS']:
-            if self.doflocs.shape[1] > 1000:
-                warn("Transforming over 1000 vertices to C_CONTIGUOUS.")
+            if self.doflocs.shape[1] > 1e3:
+                logger.warning("Transforming over 1000 vertices "
+                               "to C_CONTIGUOUS.")
             self.doflocs = np.ascontiguousarray(self.doflocs)
 
         if not self.t.flags['C_CONTIGUOUS']:
-            if self.t.shape[1] > 1000:
-                warn("Transforming over 1000 elements to C_CONTIGUOUS.")
+            if self.t.shape[1] > 1e3:
+                logger.warning("Transforming over 1000 elements "
+                               "to C_CONTIGUOUS.")
             self.t = np.ascontiguousarray(self.t)
 
+        # add boundary tags for the default cubes
+        if ((self._boundaries is None
+             and self.doflocs.shape[1] == 2 ** self.dim()
+             and self.dim() <= 2)):
+            boundaries = {}
+            # default boundary names along the dimensions
+            minnames = ['left', 'bottom', 'front']
+            maxnames = ['right', 'top', 'back']
+            for d in range(self.doflocs.shape[0]):
+                dmin = np.min(self.doflocs[d])
+                ix = self.facets_satisfying(lambda x: x[d] == dmin)
+                if len(ix) >= 1:
+                    boundaries[minnames[d]] = ix
+            for d in range(self.doflocs.shape[0]):
+                dmax = np.max(self.doflocs[d])
+                ix = self.facets_satisfying(lambda x: x[d] == dmax)
+                if len(ix) >= 1:
+                    boundaries[maxnames[d]] = ix
+            if len(boundaries) > 0:
+                self._boundaries = boundaries
+
+        # run validation
+        if self.validate and logger.getEffectiveLevel() <= logging.DEBUG:
+            self.is_valid()
+
+    def is_valid(self, raise_=False) -> bool:
+        """Perform expensive mesh validation.
+
+        Parameters
+        ----------
+            raise_: raise an exception if the mesh is invalid.
+
+        Returns
+        -------
+            bool: True if the mesh is valid.
+        """
+        logger.debug("Running mesh validation.")
+
+        # check that the shape of doflocs and t are correct
+        if self.doflocs.shape[0] != self.elem.refdom.dim():
+            msg = "Mesh.doflocs, the point array, has incorrect shape."
+            if raise_:
+                raise ValueError(msg)
+            logger.debug(msg)
+            return False
+
+        if self.t.shape[0] != self.elem.refdom.nnodes:
+            msg = "Mesh.t, the element connectivity, has incorrect shape."
+            if raise_:
+                raise ValueError(msg)
+            logger.debug(msg)
+            return False
+
+        # check that there are no duplicate points
+        tmp = np.ascontiguousarray(self.p.T)
+        p_unique = np.unique(tmp.view([('', tmp.dtype)] * tmp.shape[1]))
+        if self.p.shape[1] != p_unique.shape[0]:
+            msg = "Mesh contains duplicate vertices."
+            if raise_:
+                raise ValueError(msg)
+            logger.debug(msg)
+            return False
+
+        # check that all points are at least in some element
+        if len(np.setdiff1d(np.arange(self.p.shape[1]),
+                            np.unique(self.t))) > 0:
+            msg = "Mesh contains a vertex not belonging to any element."
+            if raise_:
+                raise ValueError(msg)
+            logger.debug(msg)
+            return False
+
+        logger.debug("Mesh validation completed with no warnings.")
+        return True
+
     def __rmatmul__(self, other):
         out = self.__matmul__(other)
         return out[1:] + out[0:1]
@@ -433,23 +516,6 @@ class Mesh:
             ]
         raise NotImplementedError
 
-    def is_valid(self) -> bool:
-        """Perform some mesh validation checks."""
-        # check that there are no duplicate points
-        tmp = np.ascontiguousarray(self.p.T)
-        if self.p.shape[1] != np.unique(tmp.view([('', tmp.dtype)]
-                                                 * tmp.shape[1])).shape[0]:
-            warn("Mesh contains duplicate vertices.")
-            return False
-
-        # check that all points are at least in some element
-        if len(np.setdiff1d(np.arange(self.p.shape[1]),
-                            np.unique(self.t))) > 0:
-            warn("Mesh contains a vertex not belonging to any element.")
-            return False
-
-        return True
-
     def __add__(self, other):
         """Join two meshes."""
         cls = type(self)
@@ -581,7 +647,7 @@ class Mesh:
     @classmethod
     def init_refdom(cls):
         """Initialize a mesh corresponding to the reference domain."""
-        return cls(cls.elem.refdom.p, cls.elem.refdom.t)
+        return cls(cls.elem.refdom.p, cls.elem.refdom.t, validate=False)
 
     def refined(self, times_or_ix: Union[int, ndarray] = 1):
         """Return a refined mesh.
@@ -594,11 +660,18 @@ class Mesh:
 
         """
         m = self
+        has_boundaries = self.boundaries is not None
+        if self.subdomains is not None:
+            logger.warning("Named subdomains invalidated by a call to "
+                           "Mesh.refined()")
         if isinstance(times_or_ix, int):
             for _ in range(times_or_ix):
                 m = m._uniform()
         else:
             m = m._adaptive(times_or_ix)
+        if has_boundaries and m.boundaries is None:
+            logger.warning("Named boundaries invalidated by a call to "
+                           "Mesh.refined()")
         return m
 
     def scaled(self, factors):
@@ -675,6 +748,39 @@ class Mesh:
             doflocs=p,
         )
 
+    def smoothed(self, fixed_nodes=None):
+        """Laplacian smoothing.
+
+        Parameters
+        ----------
+        fixed_nodes
+            A list of node indices that do not move.  By default, the boundary
+            nodes are fixed.
+
+        """
+
+        if fixed_nodes is None:
+            fixed_nodes = self.boundary_nodes()
+
+        p = np.zeros(self.doflocs.shape)
+        nv = p.shape[1]
+
+        edges = self.edges if p.shape[0] == 3 else self.facets
+        nneighbors = np.bincount(edges.reshape(-1), minlength=nv)
+
+        p += np.array([np.bincount(edges[0], pi, minlength=nv)
+                       for pi in self.doflocs[:, edges[1]]])
+        p += np.array([np.bincount(edges[1], pi, minlength=nv)
+                       for pi in self.doflocs[:, edges[0]]])
+        p /= nneighbors
+
+        p[:, fixed_nodes] = self.doflocs[:, fixed_nodes]
+
+        return replace(
+            self,
+            doflocs=p,
+        )
+
     def _uniform(self):
         """Perform a single uniform refinement."""
         raise NotImplementedError
@@ -715,7 +821,9 @@ class Mesh:
             for itr in range(len(x) - 1):
                 p = np.vstack((p, x[itr + 1].flatten()))
 
-        return cls(p, t)
+        # Always creates a mesh with duplicate verticies. Use validate=False to
+        # suppress confusing logger DEBUG messages.
+        return cls(p, t, validate=False)
 
     @staticmethod
     def build_entities(t, indices, sort=True):
@@ -799,5 +907,10 @@ class Mesh:
 
     def draw(self, *args, **kwargs):
         """Convenience wrapper for vedo."""
-        from skfem.visuals.vedo import draw
-        return draw(self, *args, **kwargs)
+        if 'visuals' in kwargs:
+            visuals = kwargs['visuals']
+            del kwargs['visuals']
+        else:
+            visuals = 'vedo'
+        mod = importlib.import_module('skfem.visuals.{}'.format(visuals))
+        return mod.draw(self, *args, **kwargs)
